/* This file is part of Eclat.
   Copyright (C) 2012 Sergey Poznyakoff.
 
   Eclat is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3, or (at your option)
   any later version.
 
   Eclat is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with Eclat.  If not, see <http://www.gnu.org/licenses/>. */

#include <config.h>
#include <string.h>
#include "libeclat.h"
#include "grecs.h"

struct pname {
	size_t i;
	char **a;
};

static int
get_param_name(void *sym, void *data)
{
	struct grecs_syment *se = sym;
	struct pname *pn = data;
	pn->a[pn->i++] = se->name;
	return 0;
}

static int
compnames(const void *a, const void *b)
{
	char * const *ac = a;
	char * const *bc = b;
	return strcmp(*ac, *bc);
}

int
eclat_query_signature(struct ec2_query *req, char *secret,
		      char **signature)
{
	char **pnames;
	size_t i, n;
	struct grecs_txtacc *acc;
	struct pname pn;
	char *str;
	char digest[20];
	size_t siglen;
	
	acc = grecs_txtacc_create();
			
	/* Collect and sort parameter names */
	n = grecs_symtab_count_entries(req->params);
	pnames = grecs_calloc(n, sizeof(pnames[0]));
	pn.i = 0;
	pn.a = pnames;
	grecs_symtab_enumerate(req->params, get_param_name, &pn);
	qsort(pnames, n, sizeof(pnames[0]), compnames);

	grecs_txtacc_grow(acc, req->verb, strlen(req->verb));
	grecs_txtacc_grow_char(acc, '\n');
	grecs_txtacc_grow(acc, req->endpoint, strlen(req->endpoint));
	grecs_txtacc_grow_char(acc, '\n');
	grecs_txtacc_grow(acc, req->uri, strlen(req->uri));
	grecs_txtacc_grow_char(acc, '\n');

	/* Append a canonicalized query string */
	for (i = 0; i < n; i++) {
		struct ec2_param *p =
			grecs_symtab_lookup_or_install(req->params, pnames[i],
						       NULL);
		if (!p)
			abort();
		if (i != 0)
			grecs_txtacc_grow_char(acc, '&');
		grecs_txtacc_grow(acc, p->name, strlen(p->name));
		grecs_txtacc_grow_char(acc, '=');
		if (p->value)
			grecs_txtacc_grow(acc, p->value, strlen(p->value));
	}
	grecs_txtacc_grow_char(acc, 0);
	str = grecs_txtacc_finish(acc, 0);

	hmac_sha1(str, strlen(str), secret, strlen(secret), digest);

	eclat_base64_encode(digest, sizeof(digest),
			    (unsigned char**) signature, &siglen);

	grecs_txtacc_free(acc);
	free(pnames);
}
